/**
  ******************************************************************************
  * @file    main.h
  * @author  MCD Application Team
  * @version V1.0.0
  * @date    31-October-2011 
  * @brief   This file contains all the functions prototypes for the main.c 
  *          file.
  ******************************************************************************
  * @attention
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; Portions COPYRIGHT 2011 STMicroelectronics</center></h2>
  ******************************************************************************
  */
/**
  ******************************************************************************
  * <h2><center>&copy; Portions COPYRIGHT 2012 Embest Tech. Co., Ltd.</center></h2>
  * @file    main.h
  * @author  CMP Team
  * @version V1.0.0
  * @date    28-December-2012
  * @brief   This file contains all the functions prototypes for the main.c 
  *          file.
  *          Modified to support the STM32F4DISCOVERY, STM32F4DIS-BB and
  *          STM32F4DIS-LCD modules. 
  ******************************************************************************
  * @attention
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, Embest SHALL NOT BE HELD LIABLE FOR ANY DIRECT, INDIRECT
  * OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING FROM THE CONTENT
  * OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE CODING INFORMATION
  * CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  ******************************************************************************
  */ 
/* Define to prevent recursive inclusion -------------------------------------*/
#ifndef __MAIN_H
#define __MAIN_H

#ifdef __cplusplus
 extern "C" {
#endif

/* Includes ------------------------------------------------------------------*/
#include "stm32f4xx.h"
#include "stm32f4x7_eth_bsp.h"
#include "stm32f4xx_gpio.h"
#include "stm32f4xx_rcc.h"
#include "stm32f4xx_syscfg.h"
#include "misc.h"
#include "stm32f4xx_can.h"
/* Exported types ------------------------------------------------------------*/
/* Exported constants --------------------------------------------------------*/

//#define USE_DHCP       /* enable DHCP, if disabled static address is used */

/////////////////////////////////////////////////////////////////////////////////
 /*	Hier die Einstellungen anpassen */
 ///////////////////////////////////////////////////////////////////////////////
/* MAC ADDRESS: MAC_ADDR0:MAC_ADDR1:MAC_ADDR2:MAC_ADDR3:MAC_ADDR4:MAC_ADDR5*/
#define MAC_ADDR0   02
#define MAC_ADDR1   80
#define MAC_ADDR2   86
#define MAC_ADDR3   12
#define MAC_ADDR4   34
#define MAC_ADDR5   01		//<--- Hier die Gruppenummer eintragen
 
/*Static IP ADDRESS: IP_ADDR0.IP_ADDR1.IP_ADDR2.IP_ADDR3 */
#define IP_ADDR0   199
#define IP_ADDR1   100
#define IP_ADDR2   100
#define IP_ADDR3   65		//<--- Hier die Gruppenummer +64 eintragen
   
/*NETMASK*/
#define NETMASK_ADDR0   255
#define NETMASK_ADDR1   255
#define NETMASK_ADDR2   255
#define NETMASK_ADDR3   0

/*Gateway Address*/
#define GW_ADDR0   IP_ADDR0
#define GW_ADDR1   IP_ADDR1
#define GW_ADDR2   IP_ADDR2
#define GW_ADDR3   1  

 /*Static IP ADDRESS for target PC*/
 #define IP_PC_ADDR0   IP_ADDR0
 #define IP_PC_ADDR1   IP_ADDR1
 #define IP_PC_ADDR2   IP_ADDR2
 #define IP_PC_ADDR3   201	//<--- Hier die PC IP-Adresse einstellen

#define DEST_PORT 5006

 //-------------------------------------------------------------------------------
 /*Bittiming*/
#define CAN_JUMPWIDTH 	CAN_SJW_2tq
#define CH1BAUD 50
#define CH2BAUD 500

#if (CH1BAUD == 500)
	#define CAN1_PRE		4

	#define CAN1_BS1 		CAN_BS1_16tq
	#define CAN1_BS2		CAN_BS2_4tq
#elif (CH1BAUD == 50)
	#define CAN1_PRE		40

	#define CAN1_BS1        CAN_BS1_16tq
	#define CAN1_BS2		CAN_BS2_4tq

#elif (CH1BAUD == 10)
	#define CAN1_PRE		320

	#define CAN1_BS1        CAN_BS1_16tq
	#define CAN1_BS2		CAN_BS2_4tq

#elif (CH1BAUD == 1000)
	#define CAN1_PRE		2

	#define CAN1_BS1        CAN_BS1_16tq
	#define CAN1_BS2		CAN_BS2_4tq
#else
	#error CH1BAUD not in supported range!
#endif


#if (CH2BAUD == 500)
	#define CAN2_PRE		4

	#define CAN2_BS1 		CAN_BS1_16tq
	#define CAN2_BS2		CAN_BS2_4tq
#elif (CH2BAUD == 50)
	#define CAN2_PRE		40

	#define CAN2_BS1        CAN_BS1_16tq
	#define CAN2_BS2		CAN_BS2_4tq

#elif (CH2BAUD == 10)
	#define CAN2_PRE		320

	#define CAN2_BS1        CAN_BS1_16tq
	#define CAN2_BS2		CAN_BS2_4tq

#elif (CH2BAUD == 1000)
	#define CAN1_PRE		2

	#define CAN2_BS1        CAN_BS1_16tq
	#define CAN2_BS2		CAN_BS2_4tq
#else
	#error CH1BAUD not in supported range!
#endif




 /////////////////////////////////////////////////////////////////////////////////
  /* Ende */
  ///////////////////////////////////////////////////////////////////////////////

/* MII and RMII mode selection, for STM324xG-EVAL Board(MB786) RevB ***********/
#define RMII_MODE

/* Exported macro ------------------------------------------------------------*/
/* Exported functions ------------------------------------------------------- */  
void Time_Update(void);



#ifdef __cplusplus
}
#endif

#endif /* __MAIN_H */


/*********** Portions COPYRIGHT 2012 Embest Tech. Co., Ltd.*****END OF FILE****/

