/**
  ******************************************************************************
  * @file    stm32f4xx_it.c
  * @author  MCD Application Team
  * @version V1.0.0
  * @date    31-October-2011
  * @brief   Main Interrupt Service Routines.
  *          This file provides template for all exceptions handler and 
  *          peripherals interrupt service routine.
  ******************************************************************************
  * @attention
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; Portions COPYRIGHT 2011 STMicroelectronics</center></h2>
  ******************************************************************************
  */
/**
  ******************************************************************************
  * <h2><center>&copy; Portions COPYRIGHT 2012 Embest Tech. Co., Ltd.</center></h2>
  * @file    stm32f4xx_it.c
  * @author  CMP Team
  * @version V1.0.0
  * @date    28-December-2012
  * @brief   Main Interrupt Service Routines.
  *          This file provides template for all exceptions handler and 
  *          peripherals interrupt service routine.    
  *          Modified to support the STM32F4DISCOVERY, STM32F4DIS-BB and
  *          STM32F4DIS-LCD modules. 
  ******************************************************************************
  * @attention
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, Embest SHALL NOT BE HELD LIABLE FOR ANY DIRECT, INDIRECT
  * OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING FROM THE CONTENT
  * OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE CODING INFORMATION
  * CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  ******************************************************************************
  */ 
/* Includes ------------------------------------------------------------------*/
#include "stm32f4xx_it.h"
#include "main.h"

/* external variables ---------------------------------------------------------*/
extern uint8_t *p_buf_can;
extern uint16_t can_work;

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
static uint8_t accept_stop=0;

/* Private function prototypes -----------------------------------------------*/
/* Private functions ---------------------------------------------------------*/

/******************************************************************************/
/*            Cortex-M4 Processor Exceptions Handlers                         */
/******************************************************************************/

/**
  * @brief   This function handles NMI exception.
  * @param  None
  * @retval None
  */
void NMI_Handler(void)
{
}

/**
  * @brief  This function handles Hard Fault exception.
  * @param  None
  * @retval None
  */
void HardFault_Handler(void)
{
  /* Go to infinite loop when Hard Fault exception occurs */
  while (1)
  {
  }
}

/**
  * @brief  This function handles Memory Manage exception.
  * @param  None
  * @retval None
  */
void MemManage_Handler(void)
{
  /* Go to infinite loop when Memory Manage exception occurs */
  while (1)
  {
  }
}

/**
  * @brief  This function handles Bus Fault exception.
  * @param  None
  * @retval None
  */
void BusFault_Handler(void)
{
  /* Go to infinite loop when Bus Fault exception occurs */
  while (1)
  {
  }
}

/**
  * @brief  This function handles Usage Fault exception.
  * @param  None
  * @retval None
  */
void UsageFault_Handler(void)
{
  /* Go to infinite loop when Usage Fault exception occurs */
  while (1)
  {
	  GPIO_SetBits(GPIOD, GPIO_Pin_14);
	  GPIO_ResetBits(GPIOD, GPIO_Pin_12);
  }
}

/**
  * @brief  This function handles SVCall exception.
  * @param  None
  * @retval None
  */
void SVC_Handler(void)
{
}

/**
  * @brief  This function handles Debug Monitor exception.
  * @param  None
  * @retval None
  */
void DebugMon_Handler(void)
{
}

/**
  * @brief  This function handles PendSVC exception.
  * @param  None
  * @retval None
  */
void PendSV_Handler(void)
{
}

/**
  * @brief  This function handles SysTick Handler.
  * @param  None
  * @retval None
  */
void SysTick_Handler(void)
{
  /* Update the LocalTime by adding SYSTEMTICK_PERIOD_MS each SysTick interrupt */
  Time_Update();
}

/******************************************************************************/
/*                 STM32F4xx Peripherals Interrupt Handlers                   */
/*  Add here the Interrupt Handler for the used peripheral(s) (PPP), for the  */
/*  available peripheral interrupt handler's name please refer to the startup */
/*  file (startup_stm32f4xx.s).                                               */
/******************************************************************************/

/**
  * @brief  This function handles PPP interrupt request.
  * @param  None
  * @retval None
  */
/*void PPP_IRQHandler(void)
{
}*/
/**
  * @brief  The ISR that receives the CAN Frame and writes it into the buffer currently used for buffering incoming Frames
  * @param  None
  * @retval None
  */
void CAN2_RX0_IRQHandler(void)
{
  CanRxMsg RxMessage;
  static uint32_t timestamp;
  uint8_t i;
  uint8_t overflowtext[]="#OVERFLOW";

  //Get the timestamp for the received frame
  timestamp= TIM_GetCounter(TIM2);

  //Get the frame
  CAN_Receive(CAN2, CAN_FIFO0, &RxMessage);

	//is this a new (empty) buffer
	if(can_work<=13)
		 {
			accept_stop=0;
		 }

	//is the buffer full and did we already insert the OVERFLOW Message?
	if(can_work>=1200 && accept_stop!=1)
	  	  {
 			//insert the Overflow Message
		  	  memcpy (&p_buf_can[can_work],overflowtext, sizeof(overflowtext));

		  	  can_work+=sizeof(overflowtext);

		  	  p_buf_can[can_work++]=0x0a;

		  	  accept_stop=1;	//for signaling that the overflow message was inserted

	  	  }

	//the buffer is not full
	if( accept_stop==0)
		  {
		  	  //which node received the frame
  	  	  	 Translate8toAscii(2,&p_buf_can[can_work]);

  	  	  	 can_work+=2;

  	  	  	 p_buf_can[can_work++]=',';

  	  	  	 //timestamp
 			 Translate32toAscii(timestamp,&p_buf_can[can_work]);

 			 can_work+=8;

 			 p_buf_can[can_work++]=',';

 			 //The frame Id
 			 Translate32toAscii(RxMessage.StdId,&p_buf_can[can_work]);

 			 can_work+=8;

 			 p_buf_can[can_work++]=',';

 			 //the frame DLC
 			 Translate8toAscii(RxMessage.DLC,&p_buf_can[can_work]);

 			 can_work+=2;

 			 p_buf_can[can_work++]=',';

 			 //copy the data of the frame into the buffer
 			 for(i=0;i<RxMessage.DLC;i++)
 				{
 					Translate8toAscii(RxMessage.Data[i],&p_buf_can[can_work]);

 					can_work+=2;

 					p_buf_can[can_work++]=',';
 				}

			p_buf_can[can_work]=0x0a;

			can_work++;
	  	  }



}




/**
  * @brief  The ISR that receives the CAN Frame and writes it into the buffer currently used for buffering incoming Frames
  * @param  None
  * @retval None
  */
void CAN1_RX1_IRQHandler(void)
{
	  CanRxMsg RxMessage;
	  static uint32_t timestamp;
	  uint8_t i;
	  uint8_t overflowtext[]="#OVERFLOW";

	  //Get the timestamp for the received frame
	  timestamp= TIM_GetCounter(TIM2);

	  //Get the frame
  	  CAN_Receive(CAN1, CAN_FIFO1, &RxMessage);

  	  //is this a new (empty) buffer
  	  if(can_work<=12)
  		  {
  		  	  accept_stop=0;
  		  }

  	  //is the buffer full and did we already insert the OVERFLOW Message?
  	  if(can_work>=1300 && accept_stop!=1)
  	  	  {
  		  	  //insert the Overflow Message
  		  	  memcpy (&p_buf_can[can_work],overflowtext, sizeof(overflowtext));

  		  	  can_work+=sizeof(overflowtext);

  		  	  p_buf_can[can_work++]=0x0a;

  		  	  accept_stop=1;//for signaling that the overflow message was inserted
  	  	  }

  	  //the buffer is not full
  	  if( accept_stop==0)
  	  	  {
  		  	  //which node received the frame
  	  	  	 Translate8toAscii(1,&p_buf_can[can_work]);

  	  	  	 can_work+=2;

  	  	  	 p_buf_can[can_work++]=',';

  	  	  	 //timestamp
 			 Translate32toAscii(timestamp,&p_buf_can[can_work]);

 			 can_work+=8;

 			 p_buf_can[can_work++]=',';

 			 //The frame Id
 			 Translate32toAscii(RxMessage.StdId,&p_buf_can[can_work]);

 			 can_work+=8;

 			 p_buf_can[can_work++]=',';

 			 //the frame DLC
 			 Translate8toAscii(RxMessage.DLC,&p_buf_can[can_work]);

 			 can_work+=2;

 			 p_buf_can[can_work++]=',';

 			//copy the data of the frame into the buffer
 			 for(i=0;i<RxMessage.DLC;i++)
 				{
 					 Translate8toAscii(RxMessage.Data[i],&p_buf_can[can_work]);

 					can_work+=2;

 					p_buf_can[can_work++]=',';
 				}

 			p_buf_can[can_work]=0x0a;

 			can_work++;
  	  	  }



}

/*********** Portions COPYRIGHT 2012 Embest Tech. Co., Ltd.*****END OF FILE****/

