import pygame as pg
from pygame.locals import *
import itertools

def channel_usage(channel_messages):
    
    # colors
    red = (255, 0, 0)
    green = (0, 255, 0)
    blue = (0, 0, 255)
    orange = (255, 100, 0)
    purple = (255, 0, 255)
    black = (0, 0, 0)
    white = (255, 255, 255)
    grey = (100, 100, 100)

    # screen parameters
    window_width = 800
    window_height = 600
    axis_height = window_height-50

    # init screen
    pg.init()
    screen = pg.display.set_mode((window_width,window_height))
    pg.display.set_caption('CAN channel usage')

    # Draw the main map
    map_width = 16000
    map_height = window_height
    main_map = pg.Surface((map_width, map_height))
    main_map = main_map.convert()
        
    def get_messages(messagelist):
        print "Reading messages"
        messages = []
        message_id_ypos= dict()        
        message_id_color= dict()  
        colorcycle=itertools.cycle([red,blue,orange,purple,black,white])
        
        # sort messages      
        all_message_ids=set([m.message_id for m in messagelist])
        sorted_ids=list(all_message_ids)
        sorted_ids.sort() 
              
        # for each message id select a color and position on screen    
        for (index,message_id) in enumerate(sorted_ids):
            message_id_ypos[message_id]=index+1.5
            message_id_color[message_id]=next(colorcycle)
        
        # seperate message information     
        for m in messagelist:
            duration=m.calculate_duration()
            endtime=m.timestamp
            starttime=endtime-duration
            ypos=message_id_ypos[m.message_id]
            color=message_id_color[m.message_id]
            intergap_duration=m.calculate_inter_frame_gap()
            messages.append(dict({'id':m.message_id,'starttime':starttime,'duration':duration,'endtime':endtime,'ypos':ypos,'color':color,'intergap_duration':intergap_duration,'payload':m.payload}))
        
        print "%d messages read" %len(messagelist)
        return messages   
      
    def get_message_ids(messages):                 
        usedID = []                
        usedID = {x['id']:x for x in messages}.values()
        return usedID
                 
    def draw_legend(surface,message_ids,map_x,map_number,messages_len):
        font = pg.font.Font(None, 20)
        percentage = 0.0
        for i in range(len(message_ids)):
            # draw message id legend
            pg.draw.line(surface,message_ids[i]['color'] , [window_width-100,15+(i)*20], [window_width-60, 15+(i)*20],10)
            text = font.render('%04X'%message_ids[i]['id'], 1,black)
            textpos = text.get_rect()
            textpos = textpos.move(window_width-50,10+(i)*20)  
            surface.blit(text, textpos)         
        
        # draw progress in %                            
        percentage = ((map_x+map_number*map_width-map_number*window_width)/float(messages_len))*100 
        text = font.render('progress: '+'%.2f'%percentage+' %',1,black)
        textpos = text.get_rect()
        textpos = textpos.move(20,10)
        surface.blit(text,textpos)
        
        # draw time on x-axis
        text = font.render('time [s]',1,black)
        textpos = text.get_rect()
        textpos = textpos.move(window_width/2 -20,window_height-25)
        surface.blit(text,textpos)
    
    
    def get_index(messages,x_scale,map_x,map_number):
        if map_number != 0:
            correction_factor = window_width*map_number
        else: 
            correction_factor = 0
        index = min(range(len(messages)), key=lambda i: abs(messages[i]['starttime']*x_scale-messages[0]['starttime']*x_scale-(map_x+map_number*map_width-correction_factor)))    
        return index

    def draw_messages(messages,x_start,x_end,x_scale,y_scale, map_number):
        box_height=0.8
        intergap_color= grey  
        count = 0     
        font = pg.font.Font(None, 20)  
        correction_factor = 0     
        main_map.fill(white) # clear map
        
        def add_box(surface,x,y,width,height,color):
            pg.draw.rect(surface, color, (x, y, width, height)) 
            
        if map_number != 0:
            correction_factor = window_width*map_number
                            
        index_start = min(range(len(messages)), key=lambda i: abs(messages[i]['starttime']*x_scale-messages[0]['starttime']*x_scale-(x_start+map_number*map_width-correction_factor)))        
        index_stop = min(range(len(messages)), key=lambda i: abs(messages[i]['endtime']*x_scale-messages[0]['starttime']*x_scale-(x_end+map_number*map_width-correction_factor))) 
    
        # draw x-axis line
        pg.draw.line(main_map, black, [(0),axis_height],[map_width, axis_height])
                                        
        for i in range (index_start,index_stop):
            # draw message box       
            add_box(main_map,
                    (messages[i]['starttime']-messages[0]['starttime'])*x_scale-map_width*map_number+correction_factor,
                    window_height-messages[i]['ypos']*y_scale,
                    messages[i]['duration']*x_scale,
                    box_height*y_scale, messages[i]['color'])
            
            # draw message gap    
            add_box(main_map,
                    (messages[i]['endtime']-messages[0]['starttime'])*x_scale-map_width*map_number+correction_factor,
                    window_height-messages[i]['ypos']*y_scale,
                    messages[i]['intergap_duration']*x_scale,
                    box_height*y_scale, intergap_color)
            
            # draw starttime of message on x-axis
            if count %(1+int(30*20000/(1+x_scale))) == 0:
                pg.draw.line(main_map, black, [(messages[i]['starttime']-messages[0]['starttime'])*x_scale-map_width*map_number+correction_factor,axis_height-3], 
                                                [(messages[i]['starttime']-messages[0]['starttime'])*x_scale-map_width*map_number+correction_factor, axis_height+3])
                text = font.render(str(messages[i]['starttime']), 1,black)
                textpos = text.get_rect()
                textpos = textpos.move((messages[i]['starttime']-messages[0]['starttime'])*x_scale-map_width*map_number+correction_factor-25,axis_height+5)
                main_map.blit(text, textpos)
            
            #draw endtime of message on x-axis
            if (messages[i+1]['starttime'] - messages[i]['endtime'])*x_scale > 70: 
                pg.draw.line(main_map, black, [(messages[i]['endtime']-messages[0]['starttime'])*x_scale-map_width*map_number+correction_factor,axis_height-3], 
                                                [(messages[i]['endtime']-messages[0]['starttime'])*x_scale-map_width*map_number+correction_factor, axis_height+3])        
                text = font.render(str(messages[i]['endtime']), 1,black)
                textpos = text.get_rect()
                textpos = textpos.move((messages[i]['endtime']-messages[0]['starttime'])*x_scale-map_width*map_number+correction_factor-25,axis_height+5)
                main_map.blit(text, textpos)
                                   
            # draw content and duration of message                                         
            if messages[i]['duration']*x_scale > 200:    
                text = font.render(str(messages[i]['payload']), 1,black)
                textpos = text.get_rect()
                textpos = textpos.move((messages[i]['starttime']-messages[0]['starttime'])*x_scale+10-map_width*map_number+correction_factor,window_height-messages[i]['ypos']*y_scale-20)
                main_map.blit(text, textpos)   
                
                text = font.render(str(messages[i]['duration']), 1,white)
                textpos = text.get_rect()
                textpos = textpos.move((messages[i]['starttime']-messages[0]['starttime']+messages[i]['duration']/2)*x_scale-map_width*map_number+correction_factor-20,
                                        window_height-messages[i]['ypos']*y_scale+box_height/2*y_scale)
                main_map.blit(text, textpos)                       
                               
            count +=1
       
         

    map_x = 0 # current x-position of the sliding window
    map_y = 0 # current y-position of the sliding window
    map_x_c = -25 # The change for x and y
    x_scale = 500 # scaling factor for x
    y_scale = 100 # scaling factor for y
    
    map_number = 0  
    messages = get_messages(channel_messages)
    message_ids = get_message_ids(messages)
    messages_len = messages[len(messages)-1]['endtime']*x_scale
    
    # initial draw of messages on screen
    draw_messages(messages,0,map_width,x_scale,y_scale,map_number)
    screen.blit(main_map, (map_x, map_y, window_width, window_height))
    pg.display.flip()

    
    # The program
    index = 0
    while True:
        for event in pg.event.get():
            pass

        key_pressed = pg.key.get_pressed()
        if key_pressed[K_LEFT]: 
            if key_pressed[K_LSHIFT]:
                map_x -= map_x_c
            else:
                map_x -= -5    

        if key_pressed[K_RIGHT]: 
            if key_pressed[K_LSHIFT]:
                map_x+= map_x_c
            else:
                map_x += -5   

        if key_pressed[K_UP]: 
            map_y -= map_x_c 

        if key_pressed[K_DOWN]: 
            map_y += map_x_c

        if (key_pressed[K_KP_PLUS] or key_pressed[K_RIGHTBRACKET]):
            
            # get index of first box in the current window before
            index = get_index(messages,x_scale,-map_x,map_number)  
           
            # adjust scaling factor     
            if key_pressed[K_LSHIFT]:
                    #x_scale += (50000*5)
                    x_scale *= 10
            else:
                    #x_scale += 1000
                    x_scale *= 2
               
            # draw messages with new scaling factor      
            draw_messages(messages,0,map_width,x_scale,y_scale, map_number)
           
            # get new position of the box
            map_x = -(messages[index]['starttime']-messages[0]['starttime'])*x_scale
            flag = 1
            map_number = 0
            while flag == 1:
                    if map_x < -(map_width-window_width):
                        map_x += map_width-window_width
                        map_number += 1
                    else:
                        flag = 0    
      
            # draw messsages again with the correct window offset 
            draw_messages(messages,0,map_width,x_scale,y_scale, map_number)
           
            # calculate new length of whole graph
            messages_len = messages[len(messages)-1]['endtime']*x_scale
        
        if (key_pressed[K_KP_MINUS] or key_pressed[K_SLASH]):  

            # get index of first box in the current window before scaling                           
            index = get_index(messages,x_scale,-map_x,map_number)    
            
            # adjust scaling factor                 
            if key_pressed[K_LSHIFT]:
                #x_scale -= (50000*5)
                x_scale /= 10
            else:
                #x_scale -= 1000
                x_scale /= 2
            if x_scale < 1:
                x_scale = 1  
                
            # draw messages with new scaling factor                              
            draw_messages(messages,0,map_width,x_scale,y_scale, map_number)
            
            # get new position of the box            
            map_x = -(messages[index]['starttime']-messages[0]['starttime'])*x_scale
            flag = 1
            map_number = 0
            while flag == 1:
                if map_x < -(map_width-window_width):
                    map_x += map_width-window_width
                    map_number += 1
                else:
                    flag = 0    
            
            # draw messsages again with the correct window offset                    
            draw_messages(messages,0,map_width,x_scale,y_scale, map_number)
            
            # calculate new length of whole graph            
            messages_len = messages[len(messages)-1]['endtime']*x_scale
          
                 
        if key_pressed[K_ESCAPE]:
            pg.quit()

        # if the window offset reaches the end of the map
        # redraw the map with new values and adjust window offset
        if map_x > 0:
            if map_number == 0:
                map_x = 0
            else:
                map_number -= 1
                map_x = -(map_width-window_width)
                draw_messages(messages,0,map_width,x_scale,y_scale, map_number)

        if map_x < -(map_width-window_width):
            map_x = 0
            map_number += 1
            draw_messages(messages,0,map_width,x_scale,y_scale, map_number)
                
        if map_y > 0:
            map_y = 0

        if map_y < -(map_height-window_height):
            map_y = -(map_height-window_height)
        
        # draw on screen
        screen.blit(main_map, (map_x, map_y, window_width, window_height))        
        draw_legend(screen,message_ids,-map_x,map_number,messages_len)
        pg.display.flip()
